@@ -36,6 +36,8 @@
 #'   For models with many observation, this leads to rather huge fitted model objects.
 #' @param ranef A flag to indicate if random effects for each level of the grouping factor(s) should be saved (default is \code{TRUE}). 
 #'   Set to \code{FALSE} to save memory. The argument has no impact on the model fitting itself.
+#' @param sample.prior A flag to indicate if samples from all specified proper priors should be additionally drawn. 
+#'   Among others, these samples can be used to calculate Bayes factors for point hypotheses. Default is \code{FALSE}. 
 #' @param fit An instance of S3 class \code{brmsfit} derived from a previous fit; defaults to \code{NA}. If \code{fit} is of class \code{brmsfit}, the compiled model associated 
 #'   with the fitted result is re-used and the arguments \code{formula}, \code{data}, \code{family}, \code{prior}, \code{addition}, \code{autocor}, \code{partial}, \code{threshold},
 #'  \code{cov.ranef}, \code{ranef}, and \code{predict} are ignored.
@@ -210,10 +212,10 @@
 #'   Families \code{gaussian}, \code{student}, and \code{cauchy} need the parameter \code{sigma} 
 #'   to account for the standard deviation of the response variable around the regression line
 #'   (not to be confused with the standard deviations of random effects). 
-#'   By default, \code{sigma} has an improper flat prior over the positiv reals. 
+#'   By default, \code{sigma} has a half cauchy prior with 'mean' 0 and 'standard deviation' 5. 
 #'   Furthermore, family \code{student} needs the parameter \code{nu} representing 
 #'   the degrees of freedom of students t distribution. 
-#'   By default, \code{nu} has prior \code{"uniform(1,60)"}. 
+#'   By default, \code{nu} has prior \code{"uniform(1,100)"}. 
 #'   Families \code{gamma} and \code{weibull} need the parameter \code{shape} 
 #'   that has a \code{"gamma(0.01,0.01)"} prior by default. For families \code{cumulative}, \code{cratio}, \code{sratio}, 
 #'   and \code{acat}, and only if \code{threshold = "equidistant"}, the parameter \code{delta} is used to model the distance
@@ -258,9 +260,9 @@
 #' @export 
 brm <- function(formula, data = NULL, family = c("gaussian", "identity"), prior = list(),
                 addition = NULL, autocor = NULL, partial = NULL, threshold = "flexible", cov.ranef = NULL, 
-                ranef = TRUE, WAIC = FALSE, predict = FALSE, fit = NA, n.chains = 2, n.iter = 2000, n.warmup = 500, 
-                n.thin = 1, n.cluster = 1, inits = "random", silent = FALSE, seed = 12345, 
-                save.model = NULL, ...) {
+                ranef = TRUE, WAIC = FALSE, predict = FALSE, sample.prior = FALSE, fit = NA, 
+                n.chains = 2, n.iter = 2000, n.warmup = 500, n.thin = 1, n.cluster = 1, inits = "random", 
+                silent = FALSE, seed = 12345, save.model = NULL, ...) {
   dots <- list(...) 
   link <- brm.link(family)
   if (n.chains %% n.cluster != 0) stop("n.chains must be a multiple of n.cluster")
@@ -281,31 +283,30 @@ brm <- function(formula, data = NULL, family = c("gaussian", "identity"), prior
                  data.name = data.name, autocor = autocor)
     x$ranef <- setNames(lapply(lapply(ee$random, brm.model.matrix, data = data), colnames), 
                         gsub("__", ":", ee$group))
-    x$pars <- brm.pars(formula, data = data, family = family, autocor = autocor, partial = partial, 
-                       threshold = threshold, ranef = ranef, WAIC = WAIC, predict = predict)
+    x$exclude <- exclude_pars(formula, ranef = ranef)
     x$data <- brm.data(formula, data = data, family = family, prior = prior, cov.ranef = cov.ranef,
                        autocor = autocor, partial = partial) 
     x$model <- stan.model(formula = x$formula, data = data, family = x$family, link = x$link, prior = prior, 
                           autocor = x$autocor, partial = x$partial, predict = predict, WAIC = WAIC,
-                          threshold = threshold, cov.ranef = names(cov.ranef), save.model = save.model)
+                          threshold = threshold, cov.ranef = names(cov.ranef),
+                          sample.prior = sample.prior, save.model = save.model)
   }  
   
   if (is.function(inits) || (is.character(inits) && !is.element(inits, c("random", "0")))) 
     inits <- replicate(n.chains, do.call(inits, list()), simplify = FALSE)
+  x$fit <- rstan::get_stanmodel(suppressMessages(rstan::stan(model_code = x$model, data = x$data, 
+                                                             chains = 0, fit = x$fit)))
   if (n.cluster > 1 || silent && n.chains > 0) {
     if (is.character(inits) || is.numeric(inits)) inits <- rep(inits, n.chains)
-    x$fit <- suppressMessages(rstan::stan(model_code = x$model, data = x$data, chains = 0, fit = x$fit, ...))
     cl <- makeCluster(n.cluster)
     clusterEvalQ(cl, require(rstan))
     clusterExport(cl = cl, c("x", "inits", "n.iter", "n.warmup", "n.thin"), envir = environment())
-    sflist <- parLapply(cl, 1:n.chains, fun = function(i)  
-      rstan::stan(fit = x$fit, data = x$data, iter = n.iter, pars = x$pars, init = inits[i],
-                  warmup = n.warmup, thin = n.thin, chains = 1, chain_id = i))
-    x$fit <- rstan::sflist2stanfit(sflist)
+    x$fit <- rstan::sflist2stanfit(parLapply(cl, 1:n.chains, fun = function(i)  
+      rstan::sampling(x$fit, data = x$data, iter = n.iter, pars = x$exclude, init = inits[i],
+                      warmup = n.warmup, thin = n.thin, chains = 1, chain_id = i, include = FALSE)))
     stopCluster(cl)
   } 
-  else x$fit <- rstan::stan(model_code = x$model, data = x$data, pars = x$pars, init = inits, 
-                            iter = n.iter, chains = n.chains, warmup = n.warmup, thin = n.thin, 
-                            fit = x$fit, ...)
-  rename.pars(x)
+  else x$fit <- rstan::sampling(x$fit, data = x$data, iter = n.iter, pars = x$exclude, init = inits,
+                                warmup = n.warmup, thin = n.thin, chains = n.chains, include = FALSE, ...)
+  return(rename.pars(x))
 }
\ No newline at end of file
